/***********************************
 * Author: Matt Christian
 * 
 * Description: Creates an on-screen 3D model and HUD, and allows
 *              player input to move around.
 * 
 * Notes: For use with XNA Development Tutorial 5
 * 
 * If you use this for a project, please give credit!
 * http://matt.insidegamer.org/xnatutorials.aspx
 * 
 * ***********************************/

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace WindowsGame1
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        //New GameModel object
        GameModel myHouse;

        //New GameCamera object
        GameCamera camera;

        //New GameSprite object
        GameSprite gameSprites;
        
        //Used to check if we need to update HUD Radar
        float previousRotation = 0.0f;

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            //Instantiate new GameModel using houseModel, remember to import model
            myHouse = new GameModel(this.Content, "Models\\houseModel");

            //Instantiate new GameCamera class
            camera = new GameCamera(graphics);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            //Instantiate new GameSprite class
            gameSprites = new GameSprite(this.Content, spriteBatch); 

            //Load Radar background sprite
            gameSprites.AddSprite("RadarBG", "RadarBG", graphics.GraphicsDevice.Viewport.Width - 100, 100, 0.0f, 0.7f);

            //Load Radar character sprite
            gameSprites.AddSprite("RadarChar", "RadarChar", graphics.GraphicsDevice.Viewport.Width - 100, 100, 0.0f, 0.7f);

            //Load 4 Xbox Buttons sprite
            gameSprites.AddSprite("4Buttons", "4Buttons", graphics.GraphicsDevice.Viewport.Width - 90, graphics.GraphicsDevice.Viewport.Height - 90, 0.0f, 1.0f);

            //Load Health background sprite
            gameSprites.AddSprite("HealthBG", "HealthBG", 140, graphics.GraphicsDevice.Viewport.Height - 100, 0.0f, 1.0f);

            //Load Health border sprite
            gameSprites.AddSprite("HealthBorder", "HealthBorder", 140, graphics.GraphicsDevice.Viewport.Height - 100, 0.0f, 1.0f);


            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            //Exit game
            if (Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();

            //Move forward
            if (Keyboard.GetState().IsKeyDown(Keys.W))
                camera.MoveForward();

            //Move Back
            if (Keyboard.GetState().IsKeyDown(Keys.S))
                camera.MoveBack();

            //Strafe Left
            if (Keyboard.GetState().IsKeyDown(Keys.A))
                camera.StrafeLeft();

            //Strafe Right
            if (Keyboard.GetState().IsKeyDown(Keys.D))
                camera.StrafeRight();

            //Turn left
            if (Keyboard.GetState().IsKeyDown(Keys.Left))
            {
                camera.turnAmt += camera.rotationSpeed;
                camera.RotateCamera();
            }

            //Turn right
            if (Keyboard.GetState().IsKeyDown(Keys.Right))
            {
                camera.turnAmt -= camera.rotationSpeed;
                camera.RotateCamera();
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            //Do we need to update the HUD?
            if (previousRotation != camera.GetRotation())
            {
                //Camera has rotated, update Radar
                gameSprites.UpdateSpriteRotation("RadarBG", "RadarBG", camera.GetRotation());
                previousRotation = camera.GetRotation();
            }

            //Set the position of the house
            myHouse.SetPosition(0.0f, 0.0f, 5.0f);

            //Render the house
            myHouse.Render(camera.GetViewMatrix(), camera.GetProjectionMatrix());

            //Draw HUD
            gameSprites.DrawSprites();

            base.Draw(gameTime);
        }
    }
}
 